<?php declare(strict_types=1);

namespace Plugin\km_rechnungen\classes;

use JTL\DB\ReturnType;
use JTL\Plugin\Helper;
use JTL\Plugin\PluginInterface;
use JTL\DB\DbInterface;

class Shoprechnungen {
    private const SHOPRECHNUNGEN_TABELLE = 'km_shoprechnungen_document_mapping';
    private DbInterface $Datenbank;
    private PluginInterface $oPlugin;
    private string $SHOPRECHNUNGEN_ROOT_ORDNER;

    public function __construct(bool $loadPlugin = true,
                                PluginInterface $oPlugin = null
    ) {
        if(!$loadPlugin && $oPlugin) {
            $this->setzePlugin($oPlugin);
        }

        $oPaths                             = $this->holePlugin()->getPaths();
        $this->Datenbank                    = \Shop::Container()->getDB();
        $this->SHOPRECHNUNGEN_ROOT_ORDNER   = PFAD_ROOT . 'plugins/' . $oPaths->getBaseDir() . '/uploads/';
    }

    public function doLog($object): void {
        $file = fopen('log.txt', 'a+');
        fwrite($file, print_r($object, true));
        fclose($file);
    }

    private function holePlugin(): PluginInterface {
        $this->oPlugin = $this->oPlugin ?? Helper::getPluginById('km_rechnungen');
        return $this->oPlugin;
    }

    private function holePluginEinstellungen(): array {
        return $this->holePlugin()->getConfig()->getAssoc();
    }

    private function holePluginEinstellung(string $cName): string {
        return $this->holePluginEinstellungen()[$cName]->value ?? '';
    }

    public function holeKey(): string {
        return $this->holePluginEinstellung('km_shoprechnungen_api_password');
    }

    public function speicherDokument(array $request): void
    {
        $oDatenbank  = $this->Datenbank;
        $bData       = $request['bData'] ?? null;
        $oUpload     = $request['upload'] ?? null;

        if ($bData) {
            $data = explode(',', base64_decode($bData));
            if (sizeof($data) > 0) {
                $oDokument                  = new \stdClass();
                $oDokument->cName           = trim($data[5]);
                $oDokument->cDateipfad      = "";
                $oDokument->cMime           = "";
                $oDokument->kKunde          = (int)$data[0];
                $oDokument->cDokumentNr     = trim($data[1]);
                $oDokument->cDokumentTyp    = trim($data[3]);
                $oDokument->dErstellt       = date('Y-m-d H:i:s', strtotime($data[2]));
                $oDokument->cBestellNr      = trim($data[4]);
                $oCheck                     = $oDatenbank->select(self::SHOPRECHNUNGEN_TABELLE, 'cDokumentNr', trim($oDokument->cDokumentNr));
                $oCheckV2                   = $oDatenbank->select(self::SHOPRECHNUNGEN_TABELLE, 'cName', trim($oDokument->cMime));
                if($oCheck === null && $oCheckV2 === null) {
                    $oDatenbank->insert(self::SHOPRECHNUNGEN_TABELLE, $oDokument);
                }
                else {
                    if($oDokument->kDokument !== null) {
                        $oDatenbank->update(self::SHOPRECHNUNGEN_TABELLE, 'kDokument', $oCheck->kDokument, $oDokument);
                    }
                }
            }
            else {
                $f = fopen('error.txt', 'a+');
                fwrite($f, "\r\n");
                fwrite($f, '['.date("Y:m:d H:i:s").']'.' Error: Fehlerhafte Daten');
                fclose($f);
            }
        }

        if($oUpload) {
            $oDokument  = $oDatenbank->select(self::SHOPRECHNUNGEN_TABELLE, 'cName', trim($oUpload['name']));
            if($oDokument) {
                $neuerPfad = $this->legeUploadPfadAn(new \DateTime((string)$oDokument->dErstellt), $oDokument->cDokumentTyp);
                $dateipfad =  str_replace(PFAD_ROOT . 'plugins/' . $this->holePlugin()->getPaths()->getBaseDir(), '' ,$neuerPfad);
                if($neuerPfad !== "") {
                    if (is_array($oUpload['name']) &&
                        sizeof($oUpload['name']) > 0
                    ) { //multiple Files
                        foreach ($oUpload as $oFile) {
                            //move file
                            move_uploaded_file($oFile['tmp_name'], $neuerPfad . '/' . $oFile['name']);
                            //save new Path
                            $oDokument->cDateipfad  = $dateipfad . '/' . $oFile['name'];
                            $oDokument->cMime       = $oFile['type'];
                            $oDatenbank->update(self::SHOPRECHNUNGEN_TABELLE, 'kDokument', $oDokument->kDokument, $oDokument);
                        }
                    } else {
                        $bMoveError = false;
                        //move file
                        try {
                            move_uploaded_file($oUpload['tmp_name'], $neuerPfad . '/' . $oUpload['name']);
                        } catch(\Exception $ex) {
                            $bMoveError = true;
                            $f = fopen('error.txt', 'a+');
                            fwrite($f, "\r\n");
                            fwrite($f, '['.date("Y:m:d H:i:s").']'.' Error:' . $ex->getMessage());
                            fclose($f);
                        }

                        if(!$bMoveError) {
                            //save new Path
                            $oDokument->cDateipfad  = $dateipfad . '/' . $oUpload['name'];
                            $oDokument->cMime       = $oUpload['type'];
                            $oDatenbank->update(self::SHOPRECHNUNGEN_TABELLE, 'kDokument', $oDokument->kDokument, $oDokument);
                        }
                    }
                }
            }
        }
    }

    private function legeUploadPfadAn(\Datetime $dt,
                                      string $cDokumentTyp
    ): string {
        $year  = $dt->format('Y');
        $month = $dt->format('m');
        $day   = $dt->format('d');

        $dokumentPfad = $this->SHOPRECHNUNGEN_ROOT_ORDNER . $this->holeDokumentTypKuerzel($cDokumentTyp) .  '/' . $year . '/' . $month . '/' . $day;

        if(!is_dir($dokumentPfad)) {
            if(mkdir($dokumentPfad, 0777, true) === false) {
                $dokumentPfad = '';
            }
        }

        return $dokumentPfad;
    }

    private function holeDokumentTypKuerzel(string $cDokumentTyp): string {
       return (array('Rechnung' => 're',
                'Lieferschein' => 'ls',
                'Angebot' => 'an',
                'Auftrag' => 'au',
                'Rechnungskorrektur' => 'rk',
                'Retoure' => 'rt',
                'Mahnung' => 'ma',
                'Gutschein' => 'gu')[$cDokumentTyp] ?? 'undefined');
    }

    private function setzePlugin(PluginInterface $oPlugin): void {
        $this->oPlugin = $oPlugin;
    }

    private function holeBestellNr(int $kBestellung): string {
        $oBestellung = $this->Datenbank->select('tbestellung', 'kBestellung', $kBestellung);
        if($oBestellung)
            return $oBestellung->cBestellNr;
        else
            return '';
    }

    public function holeDokumente(int $kKunde): array {
        return $this->Datenbank->query('select * from ' . self::SHOPRECHNUNGEN_TABELLE . ' where kKunde='.$kKunde,
            ReturnType::ARRAY_OF_OBJECTS);
    }

    public function holeDokumenteViaBestellung(int $kBestellung): array {
        return $this->Datenbank->query('select * from ' . self::SHOPRECHNUNGEN_TABELLE . ' where cBestellNr="' . $this->holeBestellNr($kBestellung). '";',
            ReturnType::ARRAY_OF_OBJECTS);
    }
}